class PySuit:
    """
    PySuit is the package interface which means that all objects inherits
    from this class. So, each object in this pacakge is an PySuit object in the
    fisrt place.
    """

    __name__ = "pysuit.PySuit()"

    def _validate_item(self, item):
        """
        Makes sure the input variable type can be processed. The main use for
        this method is to make sure we can't create nested objects from the
        package.

        Parameters
        ----------
        item: object
            The input object of any type.

        Raises
        -------
        ValueError: If `item` is `None`
        TypeError: If `item` is an `PySuit` object.
        """
        if item is None:
            raise ValueError(
                f"Can't use `None` as an element within `{self.__name__}`!!"
            )
        elif isinstance(item, PySuit):
            raise TypeError(
                f"Can't use `{self.__name__}` with \
                            `{item.__name__}`!!"
            )



# -*- coding: utf-8-*-
"""
###### Dict Collections #######
## Version 0.1 ##

# ---- List of fuctions -----
# dictsort = dict by keys and values are sorted
"""

import logging
from pysuit.interface import PySuit


class DictCollections(PySuit):
    """
    List of dict collections
    """

    __name__ = "pysuit.DictCollections()"

    def dictsort(self, dictionary, flag='keys'):
        """
        Sorting dict by keys or values
        """
        try:
            final_dict = {}
            # Sorting dict by values
            if flag.lower() == "values":
                sorted_dict = sorted(
                    list(dictionary.values()),
                    key=lambda x: (len(str(x)), str(x))
                )
                for value in sorted_dict:
                    key_result = [
                        key for key in dictionary if dictionary[key] == value
                    ]
                    final_dict[key_result[0]] = value
            # Sorting dict by keys
            else:
                sorted_dict = sorted(
                    list(dictionary.keys()),
                    key=lambda x: (len(str(x)), str(x))
                )
                final_dict = {key: dictionary[key] for key in sorted_dict}

            return final_dict

        except Exception as errors:
            logging.exception(f"error while accessing the dict: {errors}")
            raise errors



from pysuit.modules.dict_collection import DictCollections
from pysuit.modules.exclude_random import ExcludeRandom
from pysuit.modules.nested_iterator import NestedCollections
from pysuit.modules.random_datatype_collection import DataTypeCollection

__version__ = '1.0.0'


__all__ = ("DictCollections", "ExcludeRandom", "NestedCollections", "DataTypeCollection")



# -*- coding: utf-8-*-

"""
Nested Collections
"""
# ---- List of fuctions -----

from pysuit.interface import PySuit


class NestedCollections(PySuit):
    """
        Nested to single Sort
    """
    __name__ = "pysuit.NestedCollections()"
    """
        created function ===> removNested
    """
    output = []

    def nested_iterator(self, nested_element):

        """
        function used for removing nested
        """
        try:

            if len(nested_element) == 0:
                self.output = []

            if type(nested_element) == set:
                for element in nested_element:
                    if type(element) in [tuple, frozenset]:

                        self.nested_iterator(element)
                    else:
                        self.output.append(element)

            else:
                for element in nested_element:
                    if type(element) in [list, tuple, set]:
                        self.nested_iterator(element)

                    else:
                        self.output.append(element)

        except Exception as e:

            raise e

        if type(nested_element) is tuple:
            return tuple(self.output)
        elif type(nested_element) is set:
            return set(self.output)
        else:
            return self.output



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pysuit', 'pysuit.modules']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pysuit',
    'version': '1.0.0',
    'description': 'This package will resolve all small intigrities of python data structures(List,Tuple,Dictionary,Set) which are not provided by-default.',
    'long_description': '# PySuit\n\n[![Read the Docs](https://readthedocs.org/projects/py-suit/badge/?version=latest)](py-suit.rtfd.io/en/latest/) [![codecov](https://codecov.io/github/deep-tw/py-suit/branch/release/graph/badge.svg?token=WR57HD3UTR)](https://codecov.io/github/deep-tw/py-suit)\n[![PyPI version](https://badge.fury.io/py/pysuit.svg)](https://badge.fury.io/py/pysuit)\n\nA unified collection of python functions  eg dictsort,nested_iterator etc\n\n#### Purpose of the Package\n+ The purpose of the package is to provide a collection of fuction and module indices in one unified libary\n\n#### Documentation\n+ The official documentation is hosted on readthedocs.io: https://py-suit.readthedocs.io/en/latest/\n\n\n\n### Features\n+  Collection of PySuit\n   -  DictCollection\n   -  NestedCollections\n   -  ExcludeRandom\n   -  DataTypeCollection\n+  Collection of DictCollection\n   -  dictsort\n   -  etc\n+  Collection of NestedCollections\n   -  nested_iterator\n   -  etc\n+  Collection of ExcludeRandom\n   -  exclude_random\n   -  etc\n+  Collection of DataTypeCollection\n   -  fake_random_datatype\n   -  etc\n### Getting Started\nThe package can be found on pypi hence you can install it using pip\n\n#### Installation\n```bash\npip install pysuit\n```\n### Usage\nUsing the short forms or abbreviated forms of indices\n\n### DictCollection\n```python\nfrom pysuit import DictCollections\n\ndict_collection = DictCollections()\n\ndict_collection.dict_sort(dictionary , flag)\n\n```\n\n#### Examples of DictCollections\n```python\nfrom pysuit import DictCollections\n```\n```python\ndict_collection = DictCollections()\n```\n```python\ndict_collection.dict_sort({"b": 3, "a": 34, 67: "c", 1: 64} , keys)\n```\n```python\nSorted Dictionary is => {1: 64, \'a\': 34, \'b\': 3, 67: \'c\'}\n```\n### NestedCollections\n```python\nfrom pysuit import NestedCollections\n\nnested_collection = NestedCollections()\n\nnested_collection.nested_iterator(nested_list)\n\n```\n#### Examples of NestedCollections\n```python\nfrom pysuit import NestedCollections\n```\n```python\n\nnested_collection = NestedCollections()\n```\n```python\nnested_collection.nested_iterator([[0, 4], [2, 3, 4], [0, 1, 2], [0, 1, 2, 3, 4], [0, 1, 2, 3, 4]])\n```\n```python\nConverted list is => [0, 4, 2, 3, 4, 0, 1, 2, 0, 1, 2, 3, 4, 0, 1, 2, 3, 4]\n```\n### ExcludeRandom\n```python\nfrom pysuit import ExcludeRandom\n\nexclude_random = ExcludeRandom()\n\nexclude_random.exclude_random(start, stop, [exclude numbers])\n\n```\n#### Examples of ExcludeRandom\n```python\nfrom pysuit import ExcludeRandom\n```\n```python\nexclude_random = ExcludeRandom()\n```\n```python\nexclude_random.exclude_random(1, 10, [4, 5, 9])\n```\n```python\nRandom number is => 8\n```\n### DataTypeCollection\n```python\nfrom pysuit import DataTypeCollection\n\ndata_collection = DataTypeCollection()\n\ndata_collection.fake_random_datatype(datatype, length of datatype)\n\n```\n#### Examples of DataTypeCollection\n```python\nfrom pysuit import DataTypeCollection\n```\n```python\ndata_collection = DataTypeCollection()\n```\n```python\ndata_collection.fake_random_datatype(list, 10)\n```\n```python\nFake random data is => [93, 91, 9163, -1967.88203, 3, \'thousand\', -197247.03, 913, 983, \'thousand\']\n```\n\n### Contribution\nContributions are welcome\nNotice a bug let us know. Thanks\n\n\n### Author\n+ Main Maintainer: Deepak Patidar / Nitesh Yadav\n+ Team Thoughtwin\n\n### License\n+ MIT\n',
    'author': 'Deepak Patidar',
    'author_email': 'deepakp@thoughtwin.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/deep-tw/py-suit.git',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8-*-
"""
###### exclude random #######
## Version 0.1 ##

# ---- List of fuctions -----
# exclude_random = this function is work for
exluding the number and print random number.
"""
import logging
from pysuit.interface import PySuit
from random import choice


class ExcludeRandom(PySuit):
    """
    except exclude generate random number.
    """
    __name__ = "pysuit.ExcludeRandom()"

    def exclude_random(self, start, stop, number_of_excludes):
        """
        this function is work for excluding the number given by
        the user and generate a random number between the range.

        """
        try:
            exclude_number_list = []
            if stop > start and len(number_of_excludes) <= ((stop - start) - 1):
                for number in number_of_excludes:
                    if (number >= start) and (number <= stop):
                        exclude_number_list.append(number)
                    else:
                        raise ValueError("Enter valid number belongs start and stop range")
            else:
                raise ValueError("please enter the valid number which belongs to range")

            result = choice(list(set(range(start, stop)) - set(exclude_number_list)))
            return result

        except Exception as errors:
            logging.exception(f"error while accessing the exclude_random: {errors}")
            raise errors



# -*- coding: utf-8-*-
"""
###### ganerate randam datatype Collections #######
## Version 0.1 ##

# ---- List of fuctions -----
# dictsort = dict by keys and values are sorted
"""
import random
import logging
from faker import Faker
from pysuit.interface import PySuit


class DataTypeCollection(PySuit):
    """
    List of fake datatype generator collections
    """

    __name__ = "pysuit.DataTypeCollection()"

    def fake_random_datatype(self, data_type, size):
        """
        Sorting dict by keys or values
        """
        try:
            fake = Faker()
            random_list = [fake.word(), fake.pyfloat(), fake.random_int()]

            # genrate fake random dict
            if data_type is dict:
                fake_dict = {}
                for lenght in range(size):
                    value = random.choice(random_list)
                    key = fake.random_int()
                    fake_dict.update({key: value})
                return fake_dict

            # generate fake random list
            if data_type in [list, tuple]:
                fake_list = []
                for lenght in range(size):
                    item = random.choice(random_list)
                    fake_list.append(item)

                # generate fake random tuple
                if data_type is tuple:
                    return tuple(fake_list)

                return fake_list

            # generate fake random set
            if data_type is set:
                fake_set = set()
                for lenght in range(size):
                    item = fake.random_int()
                    fake_set.add(item)
                return fake_set

        except Exception as errors:
            logging.exception(f"error while accessing the dict: {errors}")
            raise errors
